#include "bintree.h"


/* Allocate memory, exit with an error-message, if no memory 
   available */ 

void* SecureMalloc(size_t size)
{
   void* res = malloc(size);

   if(!res)
   {
      printf("malloc() failure -- out of memory?");
      exit(EXIT_FAILURE);
   }
   return res;
 }

/* Insert an integer key into a binary tree. */

void BinTreeInsert(BinTree_p *root, int key)
{
   if(!(*root))
   {
      *root =BinTreeCellAlloc();
      (*root)->lchild = NULL;
      (*root)->rchild = NULL;
      (*root)->key = key;
   }
   else if(key < (*root)->key)
   {
      BinTreeInsert(&((*root)->lchild), key);
   }
   else if(key > (*root)->key)
   {
      BinTreeInsert(&((*root)->rchild), key);
   }
   /* Otherwise key is already in tree! */
}


/* Free all memory taken up by a binary tree */

void BinTreeFree(BinTree_p root)
{
   if(root)
   {
      BinTreeFree(root->lchild);
      BinTreeFree(root->rchild);
      free(root);
   }
}

/* Print tree in natural order */

void BinTreePrintNatural(BinTree_p root)
{
   if(root)
   {
      BinTreePrintNatural(root->lchild);
      printf("%3d ", root->key);
      BinTreePrintNatural(root->rchild);
   }
}

/* Print tree in preorder */

void BinTreePrintPreorder(BinTree_p root)
{
   if(root)
   {
      printf("%3d ", root->key);
      BinTreePrintPreorder(root->lchild);
      BinTreePrintPreorder(root->rchild);
   }
}

/* Print tree in postorder */

void BinTreePrintPostorder(BinTree_p root)
{
   if(root)
   {
      BinTreePrintPostorder(root->lchild);
      BinTreePrintPostorder(root->rchild);
      printf("%3d ", root->key);
   }
}
